/* diagram.cc
 * Copyright (C) 2001 The University of Newcastle upon Tyne
 *
 * Written by Paul Coates (Paul.Coates@ncl.ac.uk)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <math.h>
#include "gui.h"
#include "diagram.h"
#include "network.h"
#include "interface.h"
#include "device.h"
#include "list.h"
#include "data.h"
#include "lcomputer.xpm"
#include "lserver.xpm"
#include "lprinter.xpm"
#include "lswitch.xpm"
#include "lrouter.xpm"
#include "lcloud.xpm"

Diagram::Diagram(GtkWidget *w)
{
	window = w;

	alloc_color(red, 65535, 0, 0);
	alloc_color(green, 0, 65535, 0);
	alloc_color(blue, 0, 0, 65535);

	alloc_color(darkred, 32767, 0, 0);
	alloc_color(darkgreen, 0, 32767, 0);
	alloc_color(darkblue, 0, 0, 32767);

	alloc_color(yellow, 65535, 65535, 0);
	alloc_color(cyan, 0, 65535, 65535);
	alloc_color(magenta, 65535, 0, 65535);

	alloc_color(darkyellow, 32767, 32767, 0);
	alloc_color(darkcyan, 0, 32767, 32767);
	alloc_color(darkmagenta, 32767, 0, 32767);

	alloc_color(lightgray, 49151, 49151, 49151);
	alloc_color(gray, 32767, 32767, 32767);
	alloc_color(darkgray, 16383, 16383, 16383);
	alloc_color(bggray, 62343, 63343, 64343);

	alloc_color(black, 0, 0, 0);
	alloc_color(white, 65535, 65535, 65535);
	alloc_color(orange, 65535, 32767, 32767);
	alloc_color(purple, 32767, 0, 65535);

	gc = gdk_gc_new(window->window);
	gc2 = gdk_gc_new(window->window);
	gc3 = gdk_gc_new(window->window);
	points = new GdkPoint[4];

	lserver.load(w, lserver_xpm);
	lmachine.load(w, lcomputer_xpm);
	lswitch.load(w, lswitch_xpm);
	lrouter.load(w, lrouter_xpm);
	lprinter.load(w, lprinter_xpm);
	lcloud.load(w, lcloud_xpm);

	showlabels = '1';
	showcolourlines = '0';
	showleafnetworks = '1';
}

Diagram::~Diagram()
{
	if (gc) gdk_gc_unref(gc);
	if (gc2) gdk_gc_unref(gc2);
	if (gc3) gdk_gc_unref(gc3);
	delete points;
}

void Diagram::scale_images(float x, float y)
{
	lserver.scale(int(96/x), int(96/y));
	lmachine.scale(int(96/x), int(96/y));
	lswitch.scale(int(192/x), int(192/y));
	lrouter.scale(int(192/x), int(192/y));
	lprinter.scale(int(96/x), int(96/y));
	lcloud.scale(int(96/x), int(96/y));
}

void Diagram::interface_color(GdkGC *gc, int type)
{
	switch (type)
	{
		case 6:  gdk_gc_set_foreground(gc, &darkgreen); break;
		case 15: gdk_gc_set_foreground(gc, &blue); break;
		case 18:
		case 22:
		case 23:
		case 28: gdk_gc_set_foreground(gc, &darkred); break;
		case 37:
		case 39:
		case 49:
		case 53:
		case 59: gdk_gc_set_foreground(gc, &yellow); break;
		case 1:
		case 24:
		default: gdk_gc_set_foreground(gc, &darkblue);
	}
}

/*
void Diagram::network_links(Data *data)
{
	Link *ln, *ln2;
	Network *n;
	Device *r;
	Interface *in;
	long tx, ty;

	for (ln=data->networks.get_head(); ln!=NULL; ln=ln->next)
	{
		n = (Network*) ln->item;
		if (n->interfaces.size() > 1)
		{
			tx = ty = 0;
			for (ln2=n->interfaces.get_head(); ln2!=NULL; ln2=ln2->next)
			{
				in = (Interface*) ln2->item;
				r = in->device;
				tx += r->x;
				ty += r->y;
			}
			n->x = tx/n->interfaces.size();
			n->y = ty/n->interfaces.size();
		}
	}
}
*/
/*
void Diagram::autobunch_leaves(Data *data)
{
}
*/
void Diagram::draw(GdkPixmap *canvas, Data *data, float scale, long xp, long yp, int w, int h, int pp)
{
	float tx, ty, fntsz;
	Network *n;
	Device *r;
	Machine *m;
	Link *ln, *ln2;
	long width, height;
	Interface *in;
	GdkFont *fnt, *fnt2;
	char fntstr[256], buf[64];
	int i, j, ww, labelx, labely;

	if (data==NULL) return;

	fntsz = 112/scale;	// 112 is the MAX point size when fully zoomed in
//	if (fntsz<6) showlabels = 0;
	sprintf(fntstr, "-urw-helvetica-bold-r-normal-*-*-%d0-*-*-p-*-iso8859-2",
		(int)fntsz);
	fnt = gdk_font_load(fntstr);

	fntsz = 56/scale;	// half the size
//	if (fntsz<6) showlabels = 0;
	sprintf(fntstr, "-urw-helvetica-bold-r-normal-*-*-%d0-*-*-p-*-iso8859-2",
		(int)fntsz);
	fnt2 = gdk_font_load(fntstr);


	width = w;
	height = h;

// first we need to scale everything to fit

	scale_images(scale, scale);
	tx = xp/scale; ty=yp/scale;
	for (ln=data->devices.get_head(); ln!=NULL; ln=ln->next)
	{
		r = (Device*) ln->item;
		r->rx = (r->x/scale)+(width/2)-tx;
		r->ry = (r->y/scale)+(height/2)-ty;
	}
	for (ln=data->networks.get_head(); ln!=NULL; ln=ln->next)
	{
		n = (Network*) ln->item;
		n->rx = (n->x/scale)+(width/2)-tx;
		n->ry = (n->y/scale)+(height/2)-ty;
	}
	for (ln=data->machines.get_head(); ln!=NULL; ln=ln->next)
	{
		m = (Machine*) ln->item;
		m->rx = (m->x/scale)+(width/2)-tx;
		m->ry = (m->y/scale)+(height/2)-ty;
	}

//	gdk_gc_set_foreground(gc, &red);
//	gdk_gc_set_line_attributes(gc, 3, GDK_LINE_ON_OFF_DASH,
//		GDK_CAP_BUTT, GDK_JOIN_MITER);


	// clear window
	if (pp==0)	// pp = print preview
	{
		gdk_gc_set_foreground(gc, &bggray);
		gdk_draw_rectangle(canvas, gc, TRUE, 0, 0, w, h);
	}

	// line width == 6 when zoomed in & 1 when scale == 6
	gdk_gc_set_line_attributes(gc, int(12/scale), GDK_LINE_SOLID,
		GDK_CAP_BUTT, GDK_JOIN_MITER);



	// draw connecting lines
	gdk_gc_set_foreground(gc, &black);
	for (ln=data->networks.get_head(); ln!=NULL; ln=ln->next)
	{
		n = (Network*) ln->item;
		if (n->show=='1')
		{
			if ((n->view=='p') && (n->interfaces.size()==2))
			{
				draw_ptop(canvas, scale, n);
			}else
			if (n->view=='f')
			{
				draw_fddi(canvas, scale, n);
			}else
			if (n->view=='b')
			{
				draw_bus(canvas, scale, n, width, height, xp, yp);
			}else
			if (n->view=='s')
			{
				for (ln2=n->interfaces.get_head(); ln2!=NULL; ln2=ln2->next)
				{
					in = (Interface*) ln2->item;
					r = in->device;
					if (r->show=='1')
					{
						if (showcolourlines=='1')
							interface_color(gc, in->type);
						else
							gdk_gc_set_foreground(gc, &black);
						gdk_draw_line(canvas, gc,
							gint(n->rx), gint(n->ry), gint(r->rx), gint(r->ry));
					}
				}
				if (n->group!='b') // draw lines for snake bus elsewhere
				{
					for (ln2=n->machines.get_head(); ln2!=NULL; ln2=ln2->next)
					{
						m = (Machine*) ln2->item;
						if (m->show=='1')
						{
							gdk_gc_set_foreground(gc, &black);
							gdk_draw_line(canvas, gc,
								gint(n->rx), gint(n->ry), gint(m->rx), gint(m->ry));
						}
					}
				}
			}

/*
			// now draw lines to machines
			for (ln2=n->machines.get_head(); ln2!=NULL; ln2=ln2->next)
			{
				m = (Machine*) ln2->item;
				if (m->show=='1')
				{
					gdk_gc_set_foreground(gc, &black);
					gdk_draw_line(canvas, gc,
						gint(n->rx), gint(n->ry), gint(m->rx), gint(m->ry));
				}
			}
*/
		}
	}

	// draw network clouds
	for (ln=data->networks.get_head(); ln!=NULL; ln=ln->next)
	{
		n = (Network*) ln->item;
		if (n->show=='1')
		{
			if (n->view=='s')
			{
				// draw little circle
				gdk_gc_set_foreground(gc, &blue);
				if (n->selected=='1')
					gdk_gc_set_foreground(gc, &darkblue);
				if (n->machines.size()==0)
				{
					gdk_gc_set_foreground(gc, &cyan);
					if (n->selected=='1')
						gdk_gc_set_foreground(gc, &darkcyan);
				}
				gdk_draw_arc(canvas, gc, TRUE,
					int(n->rx-(n->w/(2*scale))), int(n->ry-(n->h/(2*scale))),
					int(n->w/scale), int(n->h/scale), 0, 360*64);
				lcloud.draw(canvas,
					int(n->rx-(n->w/(2*scale))),
					int(n->ry-(n->h/(2*scale))));
				// draw black edge
//				gdk_gc_set_foreground(gc3, &black);
//				gdk_draw_arc(canvas, gc3, FALSE,
//					int(n->rx-(n->w/(2*scale))), int(n->ry-(n->h/(2*scale))),
//					int(n->w/scale), int(n->h/scale), 0, 360*64);
			}
		}
	}

	// draw machines
	for (ln=data->machines.get_head(); ln!=NULL; ln=ln->next)
	{
		m = (Machine*) ln->item;
		if ((m->show=='1') && (m->network!=NULL) && (m->network->show=='1'))
		{
			gdk_gc_set_foreground(gc, &blue);
			if (m->selected=='1')
				gdk_gc_set_foreground(gc, &darkblue);
			gdk_draw_rectangle(canvas, gc, TRUE,
				int(m->rx-(m->w/(2*scale))), int(m->ry-(m->h/(2*scale))),
				int(m->w/scale), int(m->h/scale));
			lmachine.draw(canvas,
				int(m->rx-(m->w/(2*scale))),
				int(m->ry-(m->h/(2*scale))));
			if ((pp==0) && (showlabels=='1') && (fntsz>=6))
			{
				strcpy(buf, m->ip.getstring());
				ww = gdk_text_width(fnt2, buf, strlen(buf));
				labelx = (int) (m->rx-(ww/2));
				labely = (int) ((m->ry+((m->h/2)-4)/scale)+fnt->ascent);
				gdk_gc_set_foreground(gc, &white);
				gdk_draw_text(canvas, fnt2, gc, labelx-1, labely-1,
					buf, strlen(buf));
				gdk_draw_text(canvas, fnt2, gc, labelx+1, labely-1,
					buf, strlen(buf));
				gdk_draw_text(canvas, fnt2, gc, labelx+1, labely+1,
					buf, strlen(buf));
				gdk_draw_text(canvas, fnt2, gc, labelx-1, labely+1,
					buf, strlen(buf));
				gdk_gc_set_foreground(gc, &black);
				gdk_draw_text(canvas, fnt2, gc, labelx, labely,
					buf, strlen(buf));
			}
		}
	}

	// draw routers & switches
	for (ln=data->devices.get_head(); ln!=NULL; ln=ln->next)
	{
		r = (Device*) ln->item;
		if (r->show=='1')
		{
			// draw main circle
			if (r->status=='1')
			{
				gdk_gc_set_foreground(gc, &green);
				if (r->selected=='1')
					gdk_gc_set_foreground(gc, &darkgreen);
			}else
			{
				gdk_gc_set_foreground(gc, &red);
				if (r->selected=='1')
					gdk_gc_set_foreground(gc, &darkred);
			}
			gdk_draw_arc(canvas, gc, TRUE,
				int(r->rx-(r->w/(2*scale))), int(r->ry-(r->h/(2*scale))),
				int(r->w/scale), int(r->h/scale), 0, 360*64);

			// draw graphics
			if (r->type == 's')
			{
				lswitch.draw(canvas,
					int(r->rx-(r->w/(2*scale))),
					int(r->ry-(r->h/(2*scale))));				
			}else
			if (r->type == 'r')
			{
				lrouter.draw(canvas,
					int(r->rx-(r->w/(2*scale))),
					int(r->ry-(r->h/(2*scale))));				
			}

			// draw black edge
//			gdk_gc_set_foreground(gc3, &black);
//			gdk_draw_arc(canvas, gc3, FALSE,
//				int(r->rx-(r->w/(2*scale))), int(r->ry-(r->h/(2*scale))),
//				int(r->w/scale), int(r->h/scale), 0, 360*64);

			if ((pp==0) && (showlabels=='1') && (fntsz>=6))
			{
				strcpy(buf, r->name);
				j=strlen(buf); for (i=0; i<j; i++) if (buf[i]=='.') buf[i]='\0';
				ww = gdk_text_width(fnt, buf, strlen(buf));
				labelx = (int) (r->rx-(ww/2));
				labely = (int) ((r->ry+((r->h/2)-4)/scale)+fnt->ascent);
				gdk_gc_set_foreground(gc, &white);
				gdk_draw_text(canvas, fnt, gc, labelx-1, labely-1,
					buf, strlen(buf));
				gdk_draw_text(canvas, fnt, gc, labelx+1, labely-1,
					buf, strlen(buf));
				gdk_draw_text(canvas, fnt, gc, labelx+1, labely+1,
					buf, strlen(buf));
				gdk_draw_text(canvas, fnt, gc, labelx-1, labely+1,
					buf, strlen(buf));
				gdk_gc_set_foreground(gc, &black);
				gdk_draw_text(canvas, fnt, gc, labelx, labely,
					buf, strlen(buf));
			}
		}
	}
	gdk_font_unref(fnt);
	gdk_font_unref(fnt2);
}

void Diagram::draw_ptop(GdkPixmap *canvas, float scale, Network *n)
{
	Device *d1, *d2;
	Interface *in;
	int xdist, ydist;
	double zdist, hdist, vdist;

	if (n->interfaces.size()!=2) return;

	in = (Interface*) n->interfaces.get_item(0); d1 = in->device;
	in = (Interface*) n->interfaces.get_item(1); d2 = in->device;
	if ((d1->show=='0') || (d2->show=='0')) return;

	xdist = gint(d1->rx - d2->rx);
	ydist = gint(d1->ry - d2->ry);
	zdist = sqrt((xdist*xdist)+(ydist*ydist));
	hdist = (xdist*(d1->w/(8*scale)))/zdist;
	vdist = (ydist*(d1->w/(8*scale)))/zdist;

	if (showcolourlines=='1')
		interface_color(gc, in->type);
	else
		gdk_gc_set_foreground(gc, &black);

/*
	gdk_draw_line(canvas, gc,
		gint(d1->rx), gint(d1->ry),
		gint(d1->rx - (xdist/2) + vdist), gint(d1->ry - (ydist/2) - hdist));
	gdk_draw_line(canvas, gc,
		gint(d1->rx - (xdist/2) + vdist), gint(d1->ry - (ydist/2) - hdist),
		gint(d1->rx - (xdist/2) - vdist), gint(d1->ry - (ydist/2) + hdist));
	gdk_draw_line(canvas, gc,
		gint(d1->rx - (xdist/2) - vdist), gint(d1->ry - (ydist/2) + hdist),
		gint(d2->rx), gint(d2->ry));
*/



	points[0].x = gint(d1->rx);
	points[0].y = gint(d1->ry);
	points[1].x = gint(d1->rx - (xdist/2) + vdist);
	points[1].y = gint(d1->ry - (ydist/2) - hdist);
	points[2].x = gint(d1->rx - (xdist/2) - vdist);
	points[2].y = gint(d1->ry - (ydist/2) + hdist);
	points[3].x = gint(d2->rx);
	points[3].y = gint(d2->ry);
	gdk_draw_lines(canvas, gc, points, 4);
}

void Diagram::draw_fddi(GdkPixmap *canvas, float scale, Network *n)
{
	Device *r;
	Machine *m;
	Interface *in;
	int xdist, ydist;
	double zdist, hdist, vdist;
	Link *ln;


	if (showcolourlines=='1')
		interface_color(gc, in->type);
	else
		gdk_gc_set_foreground(gc, &black);

	for (ln=n->interfaces.get_head(); ln!=NULL; ln=ln->next)
	{
		in = (Interface*) ln->item;
		r = in->device;

		if (r->show=='1')
		{
			xdist = gint(r->rx - n->rx);
			ydist = gint(r->ry - n->ry);
			zdist = sqrt((xdist*xdist)+(ydist*ydist));
			hdist = (xdist*(r->w/(scale)))/zdist;
			vdist = (ydist*(r->w/(scale)))/zdist;

			gdk_draw_line(canvas, gc,
				gint(r->rx), gint(r->ry),
				gint(n->rx + hdist), gint(n->ry + vdist));
		}
	}

	for (ln=n->machines.get_head(); ln!=NULL; ln=ln->next)
	{
		m = (Machine*) ln->item;

		if (m->show=='1')
		{
			xdist = gint(m->rx - n->rx);
			ydist = gint(m->ry - n->ry);
			zdist = sqrt((xdist*xdist)+(ydist*ydist));
			hdist = (xdist*((m->w*2)/(scale)))/zdist;
			vdist = (ydist*((m->w*2)/(scale)))/zdist;

			gdk_draw_line(canvas, gc,
				gint(m->rx), gint(m->ry),
				gint(n->rx + hdist), gint(n->ry + vdist));
		}
	}

	gdk_draw_arc(canvas, gc, FALSE,
		int(n->rx-(n->w/(2*scale))), int(n->ry-(n->h/(2*scale))),
		int((n->w)/scale), int((n->h)/scale), 0, 360*64);
}

void Diagram::draw_bus(GdkPixmap *canvas, float scale, Network *n, long width, long height, long xp, long yp)
{
	Device *r;
	Machine *m;
	Interface *in;
	int count, xmin, xmax, ymin, ymax;
	Link *ln;

	if (n->interfaces.size()<1) return;

	count=0;
	for (ln=n->interfaces.get_head(); ln!=NULL; ln=ln->next)
	{
		in = (Interface*) ln->item;
		r = in->device;

		if (r->show=='1')
		{
			if (count==0)
			{
				xmin = xmax = (int) r->x;
				ymin = ymax = (int) r->y;
			}else
			{
				if (xmax<r->x) xmax=(int)r->x;
				if (xmin>r->x) xmin=(int)r->x;
				if (ymax<r->y) ymax=(int)r->y;
				if (ymin>r->y) ymin=(int)r->y;
			}
			count++;
		}
	}

	for (ln=n->machines.get_head(); ln!=NULL; ln=ln->next)
	{
		m = (Machine*) ln->item;

		if (m->show=='1')
		{
			if (count==0)
			{
				xmin = xmax = (int) m->x;
				ymin = ymax = (int) m->y;
			}else
			{
				if (xmax<m->x) xmax=(int)m->x;
				if (xmin>m->x) xmin=(int)m->x;
				if (ymax<m->y) ymax=(int)m->y;
				if (ymin>m->y) ymin=(int)m->y;
			}
			count++;
		}
	}

	if (count==0) return;

	// at this point r still points to a device
	// set size & coordinates of bus
	if ((xmax-xmin)>(ymax-ymin))
	{
		n->w = xmax - xmin + r->w;
		n->h = (r->h/4);
		n->x = xmin + (xmax - xmin)/2;		// xmin - (r->w/2);
		// keep n->y
	}else
	{
		n->w = (r->w/4);
		n->h = ymax - ymin + r->h;
		// keep n->x
		n->y = ymin + (ymax - ymin)/2;		// ymin - (r->h/2);
	}

	// convert x -> rx & y -> ry
	n->rx = (n->x/scale)+(width/2)-(xp/scale);
	n->ry = (n->y/scale)+(height/2)-(yp/scale);

	if (showcolourlines=='1')
		interface_color(gc, in->type);
	else
		gdk_gc_set_foreground(gc, &black);

	// now draw the bus
	if ((xmax-xmin)>(ymax-ymin))
	{
		gdk_draw_line(canvas, gc,
			gint(n->rx-(n->w/(2*scale))), gint(n->ry),
			gint(n->rx+(n->w/(2*scale))), gint(n->ry));
		for (ln=n->interfaces.get_head(); ln!=NULL; ln=ln->next)
		{
			in = (Interface*) ln->item;
			r = in->device;

			if (r->show=='1')
			{
				gdk_draw_line(canvas, gc,
					gint(r->rx), gint(r->ry),
					gint(r->rx), gint(n->ry));
			}
		}
		for (ln=n->machines.get_head(); ln!=NULL; ln=ln->next)
		{
			m = (Machine*) ln->item;
			if (m->show=='1')
			{
				gdk_draw_line(canvas, gc,
					gint(m->rx), gint(m->ry),
					gint(m->rx), gint(n->ry));
			}
		}
	}else
	{
		gdk_draw_line(canvas, gc,
			gint(n->rx), gint(n->ry-(n->h/(2*scale))),
			gint(n->rx), gint(n->ry+(n->h/(2*scale))));
		for (ln=n->interfaces.get_head(); ln!=NULL; ln=ln->next)
		{
			in = (Interface*) ln->item;
			r = in->device;

			if (r->show=='1')
			{
				gdk_draw_line(canvas, gc,
					gint(r->rx), gint(r->ry),
					gint(n->rx), gint(r->ry));
			}
		}
		for (ln=n->machines.get_head(); ln!=NULL; ln=ln->next)
		{
			m = (Machine*) ln->item;
			if (m->show=='1')
			{
				gdk_draw_line(canvas, gc,
					gint(m->rx), gint(m->ry),
					gint(n->rx), gint(m->ry));
			}
		}
	}
}

