/* viewssidebar.cc
 * Copyright (C) 2001 The University of Newcastle upon Tyne
 *
 * Written by Paul Coates (Paul.Coates@ncl.ac.uk)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <pwd.h>
#include "viewsidebar.h"
#include "device.h"
#include "main.h"
#include "session.h"
#include "data.h"
#include "scheduler.h"
#include "gui.h"
#include "devicedetails.h"
#include "ipaddress.h"
#include "features.h"
#include "view.h"
#include "stick.xpm"
#include "scross.xpm"

static void ok_cb(GtkWidget *w, gpointer data)
{ ((ViewSideBar*) data)->msg.hide(); }
static void add_cb(GtkWidget *w, gpointer data)
{ ((ViewSideBar*)data)->addview(); }
static void default_cb(GtkWidget *w, gpointer data)
{ ((ViewSideBar*)data)->defaultview(); }
static void apply_cb(GtkWidget *w, gpointer data)
{ ((ViewSideBar*)data)->applyview(); }
static void store_cb(GtkWidget *w, gpointer data)
{ ((ViewSideBar*)data)->storeview(); }
static void delete_cb(GtkWidget *w, gpointer data)
{ ((ViewSideBar*)data)->del(); }
static void select_cb(GtkWidget *clist, gint row, gint col,
	GdkEventButton *e, gpointer data)
{ ((ViewSideBar*)data)->select(row); }
static void unselect_cb(GtkWidget *clist, gint row, gint col,
	GdkEventButton *e, gpointer data)
{ ((ViewSideBar*)data)->unselect(); }

ViewSideBar::ViewSideBar(Main *m)
{
  GtkWidget *label183;
  GtkWidget *hbox90;
  GtkWidget *button94;
  GtkWidget *scrolledwindow34;
  GtkWidget *hbox89;
  GtkWidget *hbox91;

	main = m;
	row = -1;

  vbox67 = gtk_vbox_new (FALSE, 2);
  gtk_widget_show (vbox67);
  gtk_container_set_border_width (GTK_CONTAINER (vbox67), 2);

  label183 = gtk_label_new ("Views");
  gtk_widget_show (label183);
  gtk_box_pack_start (GTK_BOX (vbox67), label183, FALSE, FALSE, 0);
  gtk_misc_set_alignment (GTK_MISC (label183), 0, 0.5);

  hbox90 = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (hbox90);
  gtk_box_pack_start (GTK_BOX (vbox67), hbox90, FALSE, FALSE, 0);

  entry42 = gtk_entry_new ();
  gtk_widget_show (entry42);
  gtk_box_pack_start (GTK_BOX (hbox90), entry42, TRUE, TRUE, 0);
  gtk_widget_set_usize (entry42, 1, -2);
	gtk_signal_connect(GTK_OBJECT(entry42), "activate",
		GTK_SIGNAL_FUNC(add_cb), this);

  button94 = gtk_button_new_with_label ("Add");
  gtk_widget_show (button94);
  gtk_box_pack_start (GTK_BOX (hbox90), button94, FALSE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT (button94), "clicked",
		GTK_SIGNAL_FUNC (add_cb), this);

  scrolledwindow34 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow34);
  gtk_box_pack_start (GTK_BOX (vbox67), scrolledwindow34, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow34),
		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  clist31 = gtk_clist_new (1);
  gtk_widget_show (clist31);
  gtk_container_add (GTK_CONTAINER (scrolledwindow34), clist31);
  gtk_clist_set_column_width (GTK_CLIST (clist31), 0, 16);
  gtk_clist_column_titles_hide (GTK_CLIST (clist31));
	gtk_signal_connect(GTK_OBJECT(clist31), "select_row",
		GTK_SIGNAL_FUNC(select_cb), this);
	gtk_signal_connect(GTK_OBJECT(clist31), "unselect_row",
		GTK_SIGNAL_FUNC(unselect_cb), this);

  hbox89 = gtk_hbox_new (TRUE, 4);
  gtk_widget_show (hbox89);
  gtk_box_pack_start (GTK_BOX (vbox67), hbox89, FALSE, TRUE, 0);

  button95 = gtk_button_new_with_label ("Apply");
  gtk_widget_show (button95);
  gtk_box_pack_start (GTK_BOX (hbox89), button95, FALSE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (button95), "clicked",
		GTK_SIGNAL_FUNC (apply_cb), this);

  button96 = gtk_button_new_with_label ("Store");
	gtk_widget_show (button96);
  gtk_box_pack_start (GTK_BOX (hbox89), button96, FALSE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (button96), "clicked",
		GTK_SIGNAL_FUNC (store_cb), this);

  hbox91 = gtk_hbox_new (TRUE, 4);
  gtk_widget_show (hbox91);
  gtk_box_pack_start (GTK_BOX (vbox67), hbox91, FALSE, TRUE, 0);

  button98 = gtk_button_new_with_label ("Delete");
	gtk_widget_show (button98);
  gtk_box_pack_start (GTK_BOX (hbox91), button98, FALSE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (button98), "clicked",
		GTK_SIGNAL_FUNC (delete_cb), this);

  button99 = gtk_button_new_with_label ("Default");
	gtk_widget_show (button99);
  gtk_box_pack_start (GTK_BOX (hbox91), button99, FALSE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (button99), "clicked",
		GTK_SIGNAL_FUNC (default_cb), this);

//  checkbutton14 = gtk_check_button_new_with_label ("Auto Bunch");
//	gtk_widget_show (checkbutton14);
//	gtk_box_pack_start (GTK_BOX (vbox67), checkbutton14, FALSE, TRUE, 0);
//	gtk_signal_connect (GTK_OBJECT (checkbutton14), "clicked",
//		GTK_SIGNAL_FUNC (autobunch_cb), this);

//  checkbutton15 = gtk_check_button_new_with_label ("Auto Move Links");
//  gtk_widget_show (checkbutton15);
//  gtk_box_pack_start (GTK_BOX (vbox67), checkbutton15, FALSE, TRUE, 0);
//  gtk_signal_connect (GTK_OBJECT (checkbutton15), "clicked",
//		GTK_SIGNAL_FUNC (automovelinks_cb), this);

	msg.setTitle("View");
	msg.setLabel("");
	msg.setButtons(3);
	gtk_signal_connect (GTK_OBJECT (msg.button_yes), "clicked",
		GTK_SIGNAL_FUNC (ok_cb), this);

//	autobunch = '0';
//	automovelinks = '0';
//	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbutton14),
//		(autobunch=='1')?TRUE:FALSE);
//	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbutton15),
//		(automovelinks=='1')?TRUE:FALSE);
}

ViewSideBar::~ViewSideBar()
{
}

void ViewSideBar::select(int r)
{
	row = r;
	gtk_widget_set_sensitive(button95, TRUE);
	gtk_widget_set_sensitive(button96, TRUE);
	gtk_widget_set_sensitive(button98, TRUE);
	gtk_widget_set_sensitive(button99, TRUE);
}

void ViewSideBar::unselect()
{
	gtk_widget_set_sensitive(button95, FALSE);
	gtk_widget_set_sensitive(button96, FALSE);
	gtk_widget_set_sensitive(button98, FALSE);
	gtk_widget_set_sensitive(button99, FALSE);
	row = -1;
}

void ViewSideBar::addview()
{
	char buf[256];
	View *v;

	strcpy(buf, gtk_entry_get_text(GTK_ENTRY(entry42)));
	gtk_entry_set_text(GTK_ENTRY(entry42), "");

	// if name already exists error message
	if (main->session->uniqueviewname(buf)==0)
	{
		msg.setLabel("You must enter a unique view name");
		msg.show();
		return;
	}

	v = new View();
	strcpy(v->name, buf);
	v->copy_from_topology(main->session->data);
	main->session->views.tag(v);
	if (!main->session->viewbit)
	{
		main->session->viewbit=TRUE;
		main->setTitle();
	}
	update();
}

void ViewSideBar::applyview()
{
	View *v;

	v = (View*) gtk_clist_get_row_data(GTK_CLIST(clist31), row);
	v->copy_to_topology(main->session->data);
	if (!main->session->viewbit)
	{
		main->session->viewbit=TRUE;
		main->setTitle();
	}
	main->gui->update_views();
}

void ViewSideBar::del()
{
	View *v;

	v = (View*) gtk_clist_get_row_data(GTK_CLIST(clist31), row);
	main->session->views.del(v);
	if (!main->session->viewbit)
	{
		main->session->viewbit=TRUE;
		main->setTitle();
	}
	delete v;
	update();
}

void ViewSideBar::storeview()
{
	View *v;

	v = (View*) gtk_clist_get_row_data(GTK_CLIST(clist31), row);
	v->copy_from_topology(main->session->data);
	if (!main->session->viewbit)
	{
		main->session->viewbit=TRUE;
		main->setTitle();
	}
}

void ViewSideBar::defaultview()
{
	View *v;

	v = (View*) gtk_clist_get_row_data(GTK_CLIST(clist31), row);
	strcpy(main->session->data->default_view, v->name);
	if (!main->session->viewbit)
	{
		main->session->viewbit=TRUE;
		main->setTitle();
	}
	update();
}

void ViewSideBar::update()
{
	Link *ln;
	View *v;
	gchar *data[1];
	int r = 0;

	gtk_clist_freeze(GTK_CLIST(clist31));
	gtk_clist_clear(GTK_CLIST(clist31));
	for (ln=main->session->views.get_head(); ln!=NULL; ln=ln->next)
	{
		v = (View*) ln->item;
		data[0] = strdup("");
		gtk_clist_append(GTK_CLIST(clist31), data);
		delete data[0];

		if (strcmp(v->name, main->session->data->default_view)==0)
			clist_pixtext(vbox67, GTK_CLIST(clist31), r, 0, (gchar **)stick_xpm, v->name);
		else
			clist_pixtext(vbox67, GTK_CLIST(clist31), r, 0, (gchar **)scross_xpm, v->name);

		gtk_clist_set_row_data(GTK_CLIST(clist31), r, v); r++;
	}
	gtk_widget_set_sensitive(button95, FALSE);
	gtk_widget_set_sensitive(button96, FALSE);
	gtk_widget_set_sensitive(button98, FALSE);
	gtk_widget_set_sensitive(button99, FALSE);
	gtk_clist_set_sort_column (GTK_CLIST (clist31), 0);
	gtk_clist_sort (GTK_CLIST (clist31));
	gtk_clist_thaw(GTK_CLIST(clist31));
}

