#include <stdlib.h>
#include <string.h>
#include "ipaddress.h"
#include "interface.h"
#include "network.h"

Interface::Interface()
{
	empty();
}

Interface::~Interface()
{
}

void Interface::empty()
{
	index = -1;
	mac.empty();
	speed = 0;
	type = 0;
	status = '0';
	strcpy(comment, "");
	strcpy(descr, "");
	ip_addresses.empty();
	len = 0;
	vlan = 0;
	network = NULL;
	device = NULL;
}

void Interface::print()
{
	printf("Interface index=%d, mac=%s, speed=%lu, type=%d, status=%c, descr=%s, comment=%s\n", index, mac.getstring(), speed, type, status, descr, comment);
}

void Interface::savexml(xmlNodePtr xmlnode)
{
	xmlNodePtr newxml;
	xmlNodePtr cur;
	IPAddress *tmpip;
	char buf[256];
	Link *ln;

	newxml = xmlNewChild(xmlnode, NULL, (xmlChar*)"Interface", NULL);
	sprintf(buf, "%d", index);
	xmlSetProp(newxml, (xmlChar*)"index", (xmlChar*)buf);
	if (!mac.zero())
		xmlSetProp(newxml, (xmlChar*)"mac", (xmlChar*)mac.getstring());
	sprintf(buf, "%lu", speed);
	xmlSetProp(newxml, (xmlChar*)"speed", (xmlChar*)buf);
	sprintf(buf, "%d", type);
	xmlSetProp(newxml, (xmlChar*)"type", (xmlChar*)buf);
	sprintf(buf, "%c", status);
	xmlSetProp(newxml, (xmlChar*)"status", (xmlChar*)buf);
	if (strcmp(descr, "")!=0)
		xmlSetProp(newxml, (xmlChar*)"description", (xmlChar*)descr);
	if (strcmp(comment, "")!=0)
		xmlSetProp(newxml, (xmlChar*)"comment", (xmlChar*)comment);
	for (ln=ip_addresses.get_head(); ln!=NULL; ln=ln->next)
	{
		cur = xmlNewChild(newxml, NULL, (xmlChar*)"Address", NULL);
		tmpip = (IPAddress*) ln->item;
		if (tmpip!=NULL)
			xmlSetProp(cur, (xmlChar*)"ip", (xmlChar*)tmpip->getstring());
	}
}

void Interface::loadxml(xmlNodePtr xmlnode)
{
	IPAddress *tmpip;
	xmlChar *buf;

	// check this is the correct  node
	if (!xmlnode->name || strcmp((char*)xmlnode->name, "Interface")!=0)
		return;

	// read properties
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"index");
	if (buf!=NULL) { sscanf((char*)buf, "%d", &index); free(buf); }
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"mac");
	if (buf!=NULL) { mac.setstring((char*)buf); free(buf); }
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"speed");
	if (buf!=NULL) { sscanf((char*)buf, "%lu", &speed); free(buf); }
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"type");
	if (buf!=NULL) { sscanf((char*)buf, "%d", &type); free(buf); }
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"status");
	if (buf!=NULL) { status = buf[0]; free(buf); }
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"description");
	if (buf!=NULL) { strcpy(descr, (char*)buf); free(buf); }
	buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"comment");
	if (buf!=NULL) { strcpy(comment, (char*)buf); free(buf); }

	ip_addresses.empty();
	xmlnode = xmlnode->childs;
	while (xmlnode != NULL)
	{
		if (xmlnode->name!=NULL)
		{
			if (strcmp((char*)xmlnode->name, "Address")==0)
			{
				tmpip = new IPAddress();
				buf = (xmlChar*)xmlGetProp(xmlnode, (xmlChar*)"ip");
				if (buf!=NULL) { tmpip->setstring((char*)buf); free(buf); }
				ip_addresses.tag(tmpip);
			}
		}
		xmlnode = xmlnode->next;
	}
}

